@@ -77,17 +77,20 @@ function autodetect_environment()
 		echo "Operating system: Linux"
 		export OS=linux
 	fi
-	if [[ "$JENKINS_HOME" != "" ]]; then
+	if [ "${GITHUB_ACTIONS:-false}" = "true" ]; then
+		echo "Running in Github Actions: yes"
+		export CACHE_DIR="$RUNNER_TOOL_CACHE/$GITHUB_JOB/$RUNNER_OS"
+	elif [[ "$JENKINS_HOME" != "" ]]; then
 		echo "Running in Jenkins: yes"
 		export IN_JEKINS=true
-                if [ $OS = "linux" ]; then
+		if [ $OS = "linux" ]; then
 		    export CACHE_DIR="$JENKINS_HOME/cache/$JOB_NAME/executor-$EXECUTOR_NUMBER"
-                else
-                    require_envvar WORKSPACE "$WORKSPACE"
-		    export CACHE_DIR="$WORKSPACE/cache/$JOB_NAME/executor-$EXECUTOR_NUMBER"
-                fi
+		else
+			require_envvar WORKSPACE "$WORKSPACE"
+			export CACHE_DIR="$WORKSPACE/cache/$JOB_NAME/executor-$EXECUTOR_NUMBER"
+		fi
 	else
-		echo "Running in Jenkins: no"
+		echo "Running in CI: no"
 		export IN_JENKINS=false
 		export CACHE_DIR="$PASSENGER_ROOT/.ci_cache"
 	fi
@@ -105,7 +108,20 @@ function sanity_check_environment()
 			echo "ERROR: Jenkins environment detected, but EXECUTOR_NUMBER environment variable not set." >&2
 			return 1
 		fi
-	fi
+	elif [ "${GITHUB_ACTIONS:-false}" = "true" ]; then
+		if [ -z "$GITHUB_JOB" ]; then
+			echo "ERROR: Github Actions environment detected, but GITHUB_JOB environment variable not set." >&2
+			return 1
+		else
+			export "JOB_NAME=$GITHUB_JOB"
+		fi
+		if [ -z "$GITHUB_RUN_ID" ]; then
+			echo "ERROR: Github Actions environment detected, but GITHUB_RUN_ID environment variable not set." >&2
+			return 1
+		else
+			export "EXECUTOR_NUMBER=$GITHUB_RUN_ID"
+		fi
+    fi
 }
 
 # The following is necessary to make the C++ tests work.
