@@ -1,9 +1,9 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2018 Phusion Holding B.V.
+ *  Copyright (c) 2010-2025 Asynchronous B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
- *  trademarks of Phusion Holding B.V.
+ *  trademarks of Asynchronous B.V.
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,6 @@
 
 #include <string>
 #include <vector>
-#include <utility>
 #include <boost/shared_array.hpp>
 #include <WrapperRegistry/Registry.h>
 #include <DataStructures/HashedStaticString.h>
@@ -79,7 +78,7 @@ private:
 	template<typename OptionsClass, typename StaticStringClass>
 	static vector<StaticStringClass *> getStringFields(OptionsClass &options) {
 		vector<StaticStringClass *> result;
-		result.reserve(20);
+		result.reserve(30);
 
 		result.push_back(&options.appRoot);
 		result.push_back(&options.appGroupName);
