@@ -1,9 +1,9 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2018 Phusion Holding B.V.
+ *  Copyright (c) 2011-2025 Asynchronous B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
- *  trademarks of Phusion Holding B.V.
+ *  trademarks of Asynchronous B.V.
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -216,6 +216,7 @@ private:
 		const ExceptionPtr &e);
 	void endRequestWithErrorResponse(Client **c, Request **r,
 		const SpawningKit::SpawnException &e, int statusCode);
+	const LString* customErrorPageEnabled(Request *req);
 	bool friendlyErrorPagesEnabled(Request *req);
 
 
