@@ -1,9 +1,9 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2018 Phusion Holding B.V.
+ *  Copyright (c) 2010-2025 Asynchronous B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
- *  trademarks of Phusion Holding B.V.
+ *  trademarks of Asynchronous B.V.
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -719,7 +719,7 @@ initializeNonPrivilegedWorkingObjects() {
 	wo->spawningKitContext->finalize();
 
 	UPDATE_TRACE_POINT();
-	wo->appPoolContext = boost::make_shared<ApplicationPool2::Context>();
+	wo->appPoolContext = boost::make_shared<ApplicationPool2::Context>(coreConfig->get("old_routing").asBool());
 	wo->appPoolContext->spawningKitFactory = boost::make_shared<SpawningKit::Factory>(
 		wo->spawningKitContext.get());
 	wo->appPoolContext->agentConfig = coreConfig->inspectEffectiveValues();
