@@ -2,7 +2,7 @@
 // execution/relationship.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -18,8 +18,6 @@
 #include <boost/asio/detail/config.hpp>
 #include <boost/asio/detail/type_traits.hpp>
 #include <boost/asio/execution/executor.hpp>
-#include <boost/asio/execution/scheduler.hpp>
-#include <boost/asio/execution/sender.hpp>
 #include <boost/asio/is_applicable_property.hpp>
 #include <boost/asio/query.hpp>
 #include <boost/asio/traits/query_free.hpp>
@@ -41,10 +39,9 @@ namespace execution {
 /// the calling context.
 struct relationship_t
 {
-  /// The relationship_t property applies to executors, senders, and schedulers.
+  /// The relationship_t property applies to executors.
   template <typename T>
-  static constexpr bool is_applicable_property_v =
-    is_executor_v<T> || is_sender_v<T> || is_scheduler_v<T>;
+  static constexpr bool is_applicable_property_v = is_executor_v<T>;
 
   /// The top-level relationship_t property cannot be required.
   static constexpr bool is_requirable = false;
@@ -59,11 +56,9 @@ struct relationship_t
   /// continuation of the calling context.
   struct fork_t
   {
-    /// The relationship_t::fork_t property applies to executors, senders, and
-    /// schedulers.
+    /// The relationship_t::fork_t property applies to executors.
     template <typename T>
-    static constexpr bool is_applicable_property_v =
-      is_executor_v<T> || is_sender_v<T> || is_scheduler_v<T>;
+    static constexpr bool is_applicable_property_v = is_executor_v<T>;
 
     /// The relationship_t::fork_t property can be required.
     static constexpr bool is_requirable = true;
@@ -88,11 +83,9 @@ struct relationship_t
   /// of the calling context.
   struct continuation_t
   {
-    /// The relationship_t::continuation_t property applies to executors,
-    /// senders, and schedulers.
+    /// The relationship_t::continuation_t property applies to executors.
     template <typename T>
-    static constexpr bool is_applicable_property_v =
-      is_executor_v<T> || is_sender_v<T> || is_scheduler_v<T>;
+    static constexpr bool is_applicable_property_v = is_executor_v<T>;
 
     /// The relationship_t::continuation_t property can be required.
     static constexpr bool is_requirable = true;
@@ -158,48 +151,28 @@ template <int I = 0>
 struct relationship_t
 {
 #if defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
-# if defined(BOOST_ASIO_NO_DEPRECATED)
   template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value));
-# else // defined(BOOST_ASIO_NO_DEPRECATED)
-  template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_sender<T>
-          >::type::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_scheduler<T>
-          >::type::value
-      ));
-# endif // defined(BOOST_ASIO_NO_DEPRECATED)
+  static constexpr bool is_applicable_property_v = is_executor<T>::value;
 #endif // defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
 
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_requirable = false);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_preferable = false);
+  static constexpr bool is_requirable = false;
+  static constexpr bool is_preferable = false;
   typedef relationship_t polymorphic_query_result_type;
 
   typedef detail::relationship::fork_t<I> fork_t;
   typedef detail::relationship::continuation_t<I> continuation_t;
 
-  BOOST_ASIO_CONSTEXPR relationship_t()
+  constexpr relationship_t()
     : value_(-1)
   {
   }
 
-  BOOST_ASIO_CONSTEXPR relationship_t(fork_t)
+  constexpr relationship_t(fork_t)
     : value_(0)
   {
   }
 
-  BOOST_ASIO_CONSTEXPR relationship_t(continuation_t)
+  constexpr relationship_t(continuation_t)
     : value_(1)
   {
   }
@@ -211,16 +184,14 @@ struct relationship_t
     struct type
     {
       template <typename P>
-      auto query(BOOST_ASIO_MOVE_ARG(P) p) const
+      auto query(P&& p) const
         noexcept(
           noexcept(
-            declval<typename conditional<true, T, P>::type>().query(
-              BOOST_ASIO_MOVE_CAST(P)(p))
+            declval<conditional_t<true, T, P>>().query(static_cast<P&&>(p))
           )
         )
         -> decltype(
-          declval<typename conditional<true, T, P>::type>().query(
-            BOOST_ASIO_MOVE_CAST(P)(p))
+          declval<conditional_t<true, T, P>>().query(static_cast<P&&>(p))
         );
     };
 #else // defined(BOOST_ASIO_HAS_DEDUCED_QUERY_MEMBER_TRAIT)
@@ -235,17 +206,17 @@ struct relationship_t
     struct type
     {
       template <typename P>
-      static constexpr auto query(BOOST_ASIO_MOVE_ARG(P) p)
+      static constexpr auto query(P&& p)
         noexcept(
           noexcept(
-            conditional<true, T, P>::type::query(BOOST_ASIO_MOVE_CAST(P)(p))
+            conditional_t<true, T, P>::query(static_cast<P&&>(p))
           )
         )
         -> decltype(
-          conditional<true, T, P>::type::query(BOOST_ASIO_MOVE_CAST(P)(p))
+          conditional_t<true, T, P>::query(static_cast<P&&>(p))
         )
       {
-        return T::query(BOOST_ASIO_MOVE_CAST(P)(p));
+        return T::query(static_cast<P&&>(p));
       }
     };
 #else // defined(BOOST_ASIO_HAS_DEDUCED_QUERY_STATIC_CONSTEXPR_MEMBER_TRAIT)
@@ -265,66 +236,65 @@ struct relationship_t
 #if defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT) \
   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR
+  static constexpr
   typename query_static_constexpr_member<T>::result_type
   static_query()
-    BOOST_ASIO_NOEXCEPT_IF((
-      query_static_constexpr_member<T>::is_noexcept))
+    noexcept(query_static_constexpr_member<T>::is_noexcept)
   {
     return query_static_constexpr_member<T>::value();
   }
 
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR
+  static constexpr
   typename traits::static_query<T, fork_t>::result_type
   static_query(
-      typename enable_if<
+      enable_if_t<
         !query_static_constexpr_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         !query_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         traits::static_query<T, fork_t>::is_valid
-      >::type* = 0) BOOST_ASIO_NOEXCEPT
+      >* = 0) noexcept
   {
     return traits::static_query<T, fork_t>::value();
   }
 
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR
+  static constexpr
   typename traits::static_query<T, continuation_t>::result_type
   static_query(
-      typename enable_if<
+      enable_if_t<
         !query_static_constexpr_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         !query_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         !traits::static_query<T, fork_t>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         traits::static_query<T, continuation_t>::is_valid
-      >::type* = 0) BOOST_ASIO_NOEXCEPT
+      >* = 0) noexcept
   {
     return traits::static_query<T, continuation_t>::value();
   }
 
   template <typename E,
       typename T = decltype(relationship_t::static_query<E>())>
-  static BOOST_ASIO_CONSTEXPR const T static_query_v
+  static constexpr const T static_query_v
     = relationship_t::static_query<E>();
 #endif // defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT)
        //   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
 
-  friend BOOST_ASIO_CONSTEXPR bool operator==(
+  friend constexpr bool operator==(
       const relationship_t& a, const relationship_t& b)
   {
     return a.value_ == b.value_;
   }
 
-  friend BOOST_ASIO_CONSTEXPR bool operator!=(
+  friend constexpr bool operator!=(
       const relationship_t& a, const relationship_t& b)
   {
     return a.value_ != b.value_;
@@ -332,24 +302,22 @@ struct relationship_t
 
   struct convertible_from_relationship_t
   {
-    BOOST_ASIO_CONSTEXPR convertible_from_relationship_t(relationship_t)
+    constexpr convertible_from_relationship_t(relationship_t)
     {
     }
   };
 
   template <typename Executor>
-  friend BOOST_ASIO_CONSTEXPR relationship_t query(
+  friend constexpr relationship_t query(
       const Executor& ex, convertible_from_relationship_t,
-      typename enable_if<
+      enable_if_t<
         can_query<const Executor&, fork_t>::value
-      >::type* = 0)
+      >* = 0)
 #if !defined(__clang__) // Clang crashes if noexcept is used here.
 #if defined(BOOST_ASIO_MSVC) // Visual C++ wants the type to be qualified.
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_query<const Executor&, relationship_t<>::fork_t>::value))
+    noexcept(is_nothrow_query<const Executor&, relationship_t<>::fork_t>::value)
 #else // defined(BOOST_ASIO_MSVC)
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_query<const Executor&, fork_t>::value))
+    noexcept(is_nothrow_query<const Executor&, fork_t>::value)
 #endif // defined(BOOST_ASIO_MSVC)
 #endif // !defined(__clang__)
   {
@@ -357,22 +325,20 @@ struct relationship_t
   }
 
   template <typename Executor>
-  friend BOOST_ASIO_CONSTEXPR relationship_t query(
+  friend constexpr relationship_t query(
       const Executor& ex, convertible_from_relationship_t,
-      typename enable_if<
+      enable_if_t<
         !can_query<const Executor&, fork_t>::value
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         can_query<const Executor&, continuation_t>::value
-      >::type* = 0)
+      >* = 0)
 #if !defined(__clang__) // Clang crashes if noexcept is used here.
 #if defined(BOOST_ASIO_MSVC) // Visual C++ wants the type to be qualified.
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_query<const Executor&,
-        relationship_t<>::continuation_t>::value))
+    noexcept(is_nothrow_query<const Executor&,
+        relationship_t<>::continuation_t>::value)
 #else // defined(BOOST_ASIO_MSVC)
-    BOOST_ASIO_NOEXCEPT_IF((
-      is_nothrow_query<const Executor&, continuation_t>::value))
+    noexcept(is_nothrow_query<const Executor&, continuation_t>::value)
 #endif // defined(BOOST_ASIO_MSVC)
 #endif // !defined(__clang__)
   {
@@ -382,10 +348,6 @@ struct relationship_t
   BOOST_ASIO_STATIC_CONSTEXPR_DEFAULT_INIT(fork_t, fork);
   BOOST_ASIO_STATIC_CONSTEXPR_DEFAULT_INIT(continuation_t, continuation);
 
-#if !defined(BOOST_ASIO_HAS_CONSTEXPR)
-  static const relationship_t instance;
-#endif // !defined(BOOST_ASIO_HAS_CONSTEXPR)
-
 private:
   int value_;
 };
@@ -397,14 +359,8 @@ const T relationship_t<I>::static_query_v;
 #endif // defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT)
        //   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
 
-#if !defined(BOOST_ASIO_HAS_CONSTEXPR)
 template <int I>
-const relationship_t<I> relationship_t<I>::instance;
-#endif
-
-template <int I>
-const typename relationship_t<I>::fork_t
-relationship_t<I>::fork;
+const typename relationship_t<I>::fork_t relationship_t<I>::fork;
 
 template <int I>
 const typename relationship_t<I>::continuation_t
@@ -416,35 +372,15 @@ template <int I = 0>
 struct fork_t
 {
 #if defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
-# if defined(BOOST_ASIO_NO_DEPRECATED)
   template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value));
-# else // defined(BOOST_ASIO_NO_DEPRECATED)
-  template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_sender<T>
-          >::type::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_scheduler<T>
-          >::type::value
-      ));
-# endif // defined(BOOST_ASIO_NO_DEPRECATED)
+  static constexpr bool is_applicable_property_v = is_executor<T>::value;
 #endif // defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
 
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_requirable = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_preferable = true);
+  static constexpr bool is_requirable = true;
+  static constexpr bool is_preferable = true;
   typedef relationship_t<I> polymorphic_query_result_type;
 
-  BOOST_ASIO_CONSTEXPR fork_t()
+  constexpr fork_t()
   {
   }
 
@@ -461,55 +397,61 @@ struct fork_t
 #if defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT) \
   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR
-  typename query_static_constexpr_member<T>::result_type
+  static constexpr typename query_static_constexpr_member<T>::result_type
   static_query()
-    BOOST_ASIO_NOEXCEPT_IF((
-      query_static_constexpr_member<T>::is_noexcept))
+    noexcept(query_static_constexpr_member<T>::is_noexcept)
   {
     return query_static_constexpr_member<T>::value();
   }
 
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR fork_t static_query(
-      typename enable_if<
+  static constexpr fork_t static_query(
+      enable_if_t<
         !query_static_constexpr_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         !query_member<T>::is_valid
-      >::type* = 0,
-      typename enable_if<
+      >* = 0,
+      enable_if_t<
         !traits::query_free<T, fork_t>::is_valid
-      >::type* = 0,
-      typename enable_if<
-        !can_query<T, continuation_t<I> >::value
-      >::type* = 0) BOOST_ASIO_NOEXCEPT
+      >* = 0,
+      enable_if_t<
+        !can_query<T, continuation_t<I>>::value
+      >* = 0) noexcept
   {
     return fork_t();
   }
 
   template <typename E, typename T = decltype(fork_t::static_query<E>())>
-  static BOOST_ASIO_CONSTEXPR const T static_query_v
+  static constexpr const T static_query_v
     = fork_t::static_query<E>();
 #endif // defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT)
        //   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
 
-  static BOOST_ASIO_CONSTEXPR relationship_t<I> value()
+  static constexpr relationship_t<I> value()
   {
     return fork_t();
   }
 
-  friend BOOST_ASIO_CONSTEXPR bool operator==(
-      const fork_t&, const fork_t&)
+  friend constexpr bool operator==(const fork_t&, const fork_t&)
   {
     return true;
   }
 
-  friend BOOST_ASIO_CONSTEXPR bool operator!=(
-      const fork_t&, const fork_t&)
+  friend constexpr bool operator!=(const fork_t&, const fork_t&)
   {
     return false;
   }
+
+  friend constexpr bool operator==(const fork_t&, const continuation_t<I>&)
+  {
+    return false;
+  }
+
+  friend constexpr bool operator!=(const fork_t&, const continuation_t<I>&)
+  {
+    return true;
+  }
 };
 
 #if defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT) \
@@ -523,35 +465,15 @@ template <int I = 0>
 struct continuation_t
 {
 #if defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
-# if defined(BOOST_ASIO_NO_DEPRECATED)
   template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value));
-# else // defined(BOOST_ASIO_NO_DEPRECATED)
-  template <typename T>
-  BOOST_ASIO_STATIC_CONSTEXPR(bool,
-    is_applicable_property_v = (
-      is_executor<T>::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_sender<T>
-          >::type::value
-        || conditional<
-            is_executor<T>::value,
-            false_type,
-            is_scheduler<T>
-          >::type::value
-      ));
-# endif // defined(BOOST_ASIO_NO_DEPRECATED)
+  static constexpr bool is_applicable_property_v = is_executor<T>::value;
 #endif // defined(BOOST_ASIO_HAS_VARIABLE_TEMPLATES)
 
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_requirable = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_preferable = true);
+  static constexpr bool is_requirable = true;
+  static constexpr bool is_preferable = true;
   typedef relationship_t<I> polymorphic_query_result_type;
 
-  BOOST_ASIO_CONSTEXPR continuation_t()
+  constexpr continuation_t()
   {
   }
 
@@ -569,38 +491,44 @@ struct continuation_t
 #if defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT) \
   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
   template <typename T>
-  static BOOST_ASIO_CONSTEXPR
-  typename query_static_constexpr_member<T>::result_type
+  static constexpr typename query_static_constexpr_member<T>::result_type
   static_query()
-    BOOST_ASIO_NOEXCEPT_IF((
-      query_static_constexpr_member<T>::is_noexcept))
+    noexcept(query_static_constexpr_member<T>::is_noexcept)
   {
     return query_static_constexpr_member<T>::value();
   }
 
   template <typename E,
       typename T = decltype(continuation_t::static_query<E>())>
-  static BOOST_ASIO_CONSTEXPR const T static_query_v
+  static constexpr const T static_query_v
     = continuation_t::static_query<E>();
 #endif // defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT)
        //   && defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
 
-  static BOOST_ASIO_CONSTEXPR relationship_t<I> value()
+  static constexpr relationship_t<I> value()
   {
     return continuation_t();
   }
 
-  friend BOOST_ASIO_CONSTEXPR bool operator==(
-      const continuation_t&, const continuation_t&)
+  friend constexpr bool operator==(const continuation_t&, const continuation_t&)
   {
     return true;
   }
 
-  friend BOOST_ASIO_CONSTEXPR bool operator!=(
-      const continuation_t&, const continuation_t&)
+  friend constexpr bool operator!=(const continuation_t&, const continuation_t&)
   {
     return false;
   }
+
+  friend constexpr bool operator==(const continuation_t&, const fork_t<I>&)
+  {
+    return false;
+  }
+
+  friend constexpr bool operator!=(const continuation_t&, const fork_t<I>&)
+  {
+    return true;
+  }
 };
 
 #if defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT) \
@@ -615,12 +543,7 @@ const T continuation_t<I>::static_query_v;
 
 typedef detail::relationship_t<> relationship_t;
 
-#if defined(BOOST_ASIO_HAS_CONSTEXPR) || defined(GENERATING_DOCUMENTATION)
-constexpr relationship_t relationship;
-#else // defined(BOOST_ASIO_HAS_CONSTEXPR) || defined(GENERATING_DOCUMENTATION)
-namespace { static const relationship_t&
-  relationship = relationship_t::instance; }
-#endif
+BOOST_ASIO_INLINE_VARIABLE constexpr relationship_t relationship;
 
 } // namespace execution
 
@@ -628,61 +551,19 @@ namespace { static const relationship_t&
 
 template <typename T>
 struct is_applicable_property<T, execution::relationship_t>
-  : integral_constant<bool,
-      execution::is_executor<T>::value
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_sender<T>
-          >::type::value
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_scheduler<T>
-          >::type::value
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    >
+  : integral_constant<bool, execution::is_executor<T>::value>
 {
 };
 
 template <typename T>
 struct is_applicable_property<T, execution::relationship_t::fork_t>
-  : integral_constant<bool,
-      execution::is_executor<T>::value
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_sender<T>
-          >::type::value
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_scheduler<T>
-          >::type::value
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    >
+  : integral_constant<bool, execution::is_executor<T>::value>
 {
 };
 
 template <typename T>
 struct is_applicable_property<T, execution::relationship_t::continuation_t>
-  : integral_constant<bool,
-      execution::is_executor<T>::value
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_sender<T>
-          >::type::value
-        || conditional<
-            execution::is_executor<T>::value,
-            false_type,
-            execution::is_scheduler<T>
-          >::type::value
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-    >
+  : integral_constant<bool, execution::is_executor<T>::value>
 {
 };
 
@@ -694,27 +575,27 @@ namespace traits {
 
 template <typename T>
 struct query_free_default<T, execution::relationship_t,
-  typename enable_if<
+  enable_if_t<
     can_query<T, execution::relationship_t::fork_t>::value
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept =
-    (is_nothrow_query<T, execution::relationship_t::fork_t>::value));
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept =
+    is_nothrow_query<T, execution::relationship_t::fork_t>::value;
 
   typedef execution::relationship_t result_type;
 };
 
 template <typename T>
 struct query_free_default<T, execution::relationship_t,
-  typename enable_if<
+  enable_if_t<
     !can_query<T, execution::relationship_t::fork_t>::value
       && can_query<T, execution::relationship_t::continuation_t>::value
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept =
-    (is_nothrow_query<T, execution::relationship_t::continuation_t>::value));
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept =
+    is_nothrow_query<T, execution::relationship_t::continuation_t>::value;
 
   typedef execution::relationship_t result_type;
 };
@@ -726,18 +607,18 @@ struct query_free_default<T, execution::relationship_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t,
-  typename enable_if<
+  enable_if_t<
     execution::detail::relationship_t<0>::
       query_static_constexpr_member<T>::is_valid
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef typename execution::detail::relationship_t<0>::
     query_static_constexpr_member<T>::result_type result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return execution::detail::relationship_t<0>::
       query_static_constexpr_member<T>::value();
@@ -746,22 +627,22 @@ struct static_query<T, execution::relationship_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t,
-  typename enable_if<
+  enable_if_t<
     !execution::detail::relationship_t<0>::
         query_static_constexpr_member<T>::is_valid
       && !execution::detail::relationship_t<0>::
         query_member<T>::is_valid
       && traits::static_query<T,
         execution::relationship_t::fork_t>::is_valid
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef typename traits::static_query<T,
     execution::relationship_t::fork_t>::result_type result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return traits::static_query<T,
         execution::relationship_t::fork_t>::value();
@@ -770,7 +651,7 @@ struct static_query<T, execution::relationship_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t,
-  typename enable_if<
+  enable_if_t<
     !execution::detail::relationship_t<0>::
         query_static_constexpr_member<T>::is_valid
       && !execution::detail::relationship_t<0>::
@@ -779,15 +660,15 @@ struct static_query<T, execution::relationship_t,
         execution::relationship_t::fork_t>::is_valid
       && traits::static_query<T,
         execution::relationship_t::continuation_t>::is_valid
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef typename traits::static_query<T,
     execution::relationship_t::continuation_t>::result_type result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return traits::static_query<T,
         execution::relationship_t::continuation_t>::value();
@@ -796,18 +677,18 @@ struct static_query<T, execution::relationship_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t::fork_t,
-  typename enable_if<
+  enable_if_t<
     execution::detail::relationship::fork_t<0>::
       query_static_constexpr_member<T>::is_valid
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef typename execution::detail::relationship::fork_t<0>::
     query_static_constexpr_member<T>::result_type result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return execution::detail::relationship::fork_t<0>::
       query_static_constexpr_member<T>::value();
@@ -816,7 +697,7 @@ struct static_query<T, execution::relationship_t::fork_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t::fork_t,
-  typename enable_if<
+  enable_if_t<
     !execution::detail::relationship::fork_t<0>::
         query_static_constexpr_member<T>::is_valid
       && !execution::detail::relationship::fork_t<0>::
@@ -824,14 +705,14 @@ struct static_query<T, execution::relationship_t::fork_t,
       && !traits::query_free<T,
         execution::relationship_t::fork_t>::is_valid
       && !can_query<T, execution::relationship_t::continuation_t>::value
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef execution::relationship_t::fork_t result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return result_type();
   }
@@ -839,18 +720,18 @@ struct static_query<T, execution::relationship_t::fork_t,
 
 template <typename T>
 struct static_query<T, execution::relationship_t::continuation_t,
-  typename enable_if<
+  enable_if_t<
     execution::detail::relationship::continuation_t<0>::
       query_static_constexpr_member<T>::is_valid
-  >::type>
+  >>
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 
   typedef typename execution::detail::relationship::continuation_t<0>::
     query_static_constexpr_member<T>::result_type result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value()
+  static constexpr result_type value()
   {
     return execution::detail::relationship::continuation_t<0>::
       query_static_constexpr_member<T>::value();
@@ -860,34 +741,6 @@ struct static_query<T, execution::relationship_t::continuation_t,
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_STATIC_QUERY_TRAIT)
        //   || !defined(BOOST_ASIO_HAS_SFINAE_VARIABLE_TEMPLATES)
 
-#if !defined(BOOST_ASIO_HAS_DEDUCED_STATIC_REQUIRE_TRAIT)
-
-template <typename T>
-struct static_require<T, execution::relationship_t::fork_t,
-  typename enable_if<
-    static_query<T, execution::relationship_t::fork_t>::is_valid
-  >::type>
-{
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid =
-    (is_same<typename static_query<T,
-      execution::relationship_t::fork_t>::result_type,
-        execution::relationship_t::fork_t>::value));
-};
-
-template <typename T>
-struct static_require<T, execution::relationship_t::continuation_t,
-  typename enable_if<
-    static_query<T, execution::relationship_t::continuation_t>::is_valid
-  >::type>
-{
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid =
-    (is_same<typename static_query<T,
-      execution::relationship_t::continuation_t>::result_type,
-        execution::relationship_t::continuation_t>::value));
-};
-
-#endif // !defined(BOOST_ASIO_HAS_DEDUCED_STATIC_REQUIRE_TRAIT)
-
 } // namespace traits
 
 #endif // defined(GENERATING_DOCUMENTATION)
