@@ -1,9 +1,9 @@
 # encoding: binary
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2011-2017 Phusion Holding B.V.
+#  Copyright (c) 2011-2025 Asynchronous B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
-#  trademarks of Phusion Holding B.V.
+#  trademarks of Asynchronous B.V.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -304,9 +304,9 @@ module PhusionPassenger
       end
 
       if options["process_title"] && !options["process_title"].empty?
-        $0 = options["process_title"] + ": " + options["app_group_name"]
+        rename_process "#{options["process_title"]}: #{options["app_group_name"]}"
       else
-        $0 = "Passenger App: " + options["app_group_name"]
+        rename_process "Passenger App: #{options["app_group_name"]}"
       end
 
       # If we were forked from a preloader process then clear or
@@ -315,7 +315,9 @@ module PhusionPassenger
       # database connection handles.
       if forked
         if defined?(ActiveRecord::Base)
-          if ActiveRecord::Base.respond_to?(:clear_all_connections!)
+          if ActiveRecord::Base.respond_to?(:connection_handler) && !ActiveRecord::Base.connection_handler.nil? && ActiveRecord::Base.connection_handler.respond_to?(:clear_all_connections!)
+            ActiveRecord::Base.connection_handler.clear_all_connections!
+          elsif ActiveRecord::Base.respond_to?(:clear_all_connections!)
             ActiveRecord::Base.clear_all_connections!
           elsif ActiveRecord::Base.respond_to?(:clear_active_connections!)
             ActiveRecord::Base.clear_active_connections!
@@ -539,6 +541,14 @@ module PhusionPassenger
       try_write_file("#{dir}/envvars", ENV.to_a.map { |k, v| "#{k} = #{v}" }.join("\n"))
     end
 
+    def rename_process(name)
+      if Process.respond_to?(:setproctitle)
+        Process.setproctitle(name)
+      else
+        $0 = name
+      end
+    end
+
   private
     def running_bundler(options)
       yield
